# Kong jq Plugin

[![Build Status](https://app.travis-ci.com/Kong/kong-plugin-jq.svg?token=xhjq5x37q11sqyVgYyyJ&branch=main)](https://app.travis-ci.com/Kong/kong-plugin-jq)

## Synopsis

Provides arbitrary [jq](https://stedolan.github.io/jq/) transformations to a
JSON object found in the request or response body.

## Configuration

The configuration accepts two sets of options; one for the request and another
for the response. In both cases a `jq` program string can be given, along with
some `jq` option flags and a list of media types, one of which must be found in
the `Content-Type` header in order for the `jq` program to run (defaults to
`application/json`).

In the response context, there is also the option to specify a list of status
codes, one of which must match the response status code (defaults to `200`).

Note that in the response context the entire body must be buffered in order to
be processed. This also implies that the `Content-Length` header will be
dropped if present, and the body transferred with chunked encoding.

### Example

Given the following JSON plugin configuration:

```
echo '{
  "name": "jq",
  "config": {
    "request_jq_program": ".[0]"
  }
}' | http POST :8001/services/service1/plugins
```

...if the request body contains the following JSON array:

```json
[
  { "foo": "bar" },
  { "bar": "foo" }
]
```

...then the request body will be rewritten with the results of the program,
i.e. the first object in the array:

```json
{ "foo": "bar" }
```

### Schema

#### `request_jq_program` (string)

The jq program to run. For example: `.[0] | { "X-Foo": .foo }`

* Required: at least one of `request_jq_program` or `response_jq_program` must
  be given.

#### `request_jq_options` (record)

Boolean option flags to modify the run behaviour of the jq program. For
explanations see the [lua-resty-jq
documentation](https://github.com/bungle/lua-resty-jq#filter).

* *compact_output* (default: `true`)
* *raw_output* (default: `false`)
* *join_output* (defualt: `false`)
* *ascii_output* (default: `false`)
* *sort_keys* (default: `false`)

#### `request_if_media_type` (array of strings)

A list of [media type](https://en.wikipedia.org/wiki/Media_type) strings, which
must be present in the `Content-Type` header for the request program to run.

* Required: `false`
* Default: `[ "application/json" ]`

#### `response_jq_program` (string)

As `request_jq_program` but pertaining to the response body.

#### `response_jq_options` (record)

As `request_jq_options` but pertaining to the `response_jq_program`.

#### `response_if_media_type` (array of strings)

As `request_if_media_type` but pertaining to the `response_jq_program` invocation.

#### `response_if_status_code`

A list of HTTP response status codes which must be present for the response
program to run.

* Default: `[200]`.
